#!/usr/bin/perl
#
# Latent Semantic Indexing Implementation
#  Written By John Resig
#        (jeresig@gmail.com)
#     http://csh.rit.edu/~phytar/
#

use strict;

my ( %data, %weights, @files, $stopFile, %stop );

push( @files, glob( "data/*.xml" ) );
$stopFile = "english.stop";

open( STOP, $stopFile );
foreach my $term ( <STOP> ) {
  chomp $term;
  $stop{ $term } = 1;
}
close ( STOP );

foreach my $file ( @files ) {
  $file =~ /([^\/]*).xml$/i;
  my $curid = $1;

  #print "Processing: $file\n";
  open ( FILE, $file );
  my $f = join( '', <FILE> );
  close( FILE );

  my $i = 0;
  while ( $f =~ /<item(.*?)<\/item/igs ) {
    my $in = $1;
    my $text = '';
    if ( $in =~ /title>(.*?)</is ) {
      $text .= $1;
    }
    if ( $in =~ /tion>(.*?)<\/desc/is ) {
      $text .= " $1";
    }
    $text =~ s/&lt;/</g;
    $text =~ s/&gt;/>/g;
    $text =~ s/&amp;/\&/g;
    $text =~ s/&quot;/"/g;
    $text =~ s/&nbsp;/ /g;
    parseWords( $curid . '-' . ++$i, $text );
  }
}

#print "Computing Weights\n";
foreach my $word ( keys %data ) {
  foreach my $doc ( keys %{ $data{ $word } } ) {
    $weights{ $doc }{ $word } = log10( 1 + $data{ $word }{ $doc } );
  }
}

#print "Normalising\n";
foreach my $doc ( keys %weights ) {
  my $SumTermFreq = 0;
  my $NormValue = 0;

  foreach my $word ( keys %{ $weights{ $doc } } ) {
    $SumTermFreq += $weights{ $doc }{ $word } *  $weights{ $doc }{ $word };
  }

  $NormValue = sqrt( $SumTermFreq ) / $SumTermFreq;

  foreach my $word ( keys %{ $weights{ $doc } } ) {
    $weights{ $doc }{ $word } *= $NormValue;
    $data{ $word }{ $doc } = $weights{ $doc }{ $word };
    delete $weights{ $doc }{ $word };
  }
}

%weights = ();
my $let = '';

#print "Writing data to files\n";
foreach my $word ( sort keys %data ) {
  $word =~ /^([a-z0-9])/;
  my $cur = $1;

  if ( $let ne $cur ) {
    close( FILE ) if ( $let ne '' );
    #print "Writing '$cur' data\n";
    open( FILE, ">out/$cur\.dat" );
  }

  $let = $cur;

  print FILE "$word ";

  foreach my $doc ( keys %{ $data{ $word } } ) {
    print FILE "$doc $data{ $word }{ $doc } ";
  }

  print FILE "\n";
}

close( FILE );

sub parseWords {
  my ( $id, $words ) = @_;

  print "$id\n";

  $words =~ s/<!\[CDATA\[//gi;
  $words =~ s/\]\]>//g;
  $words =~ s/<[^>]*?>/ /g;

  while ( $words =~ s/([a-zA-Z0-9']+)// ) {
    my $word = lc( $1 );
    $word =~ s/^'//;
    $word =~ s/'$//;
    if ( ( ! exists $stop{ $word } ) && $word =~ /[a-z]/ ) {
      #print "$word $id\n";
      $data{ $word }{ $id } += 1;
    }
  } 
}

sub log10 {
  log( $_[ 0 ] ) / log( 10 );
}

